#include "CGm32.h"
#include <stdio.h>

#define SAFE_FREE(p)	if (p){free(p);p=NULL;}

#define CHIP_SIZE			32	// }bv`bv̂P`bv̈ӂ̒i32 Œj
#define CHIP_IMG_COL		16	// }bv`bv̏c̐i16 Œj
#define CHIP_IMG_ROW		16	// }bv`bv̉̐i16 Œj

#define HIT_SIZE			32	// 蔻TCY i32 Œj

#define BYTE_MASK_BG		0x000000FF
#define BYTE_MASK_FLG		0x0000FF00
#define BYTE_MASK_LYER		0x00FF0000
#define BYTE_MASK_UNDEF		0xFF000000

#define BIT_MASK_HIT16_0	0x00000100
#define BIT_MASK_HIT16_1	0x00000200
#define BIT_MASK_HIT16_2	0x00000400
#define BIT_MASK_HIT16_3	0x00000800
#define BIT_MASK_PRI16_0	0x00001000
#define BIT_MASK_PRI16_1	0x00002000
#define BIT_MASK_PRI16_2	0x00004000
#define BIT_MASK_PRI16_3	0x00008000

#define BYTE_SHIFT_BG		0
#define BYTE_SHIFT_FLG		8
#define BYTE_SHIFT_LYER		16
#define BYTE_SHIFT_UNDEF	24

// CGm32 ̃RXgN^
CGm32::CGm32()
{
	m_MapSizeX	= 0;
	m_MapSizeY	= 0;
	m_MapWidth	= 10;
	m_MapHeight	= 8;
	m_pMapData	= NULL;
}// CGm32

//	}bvf[^i.binjǂݍ
BOOL CGm32::LoadMapData(TCHAR *filename)
{
	FILE *fp = _tfopen(filename, _T("rb"));
	if (fp == NULL) {
		return FALSE;
	}
	
	// WAcW̓ǂݍ
	m_MapSizeX  = fgetc(fp);
	m_MapSizeX |= fgetc(fp) << 8;
	m_MapSizeX |= fgetc(fp) << 16;
	m_MapSizeX |= fgetc(fp) << 24;
	
	m_MapSizeY  = fgetc(fp);
	m_MapSizeY |= fgetc(fp) << 8;
	m_MapSizeY |= fgetc(fp) << 16;
	m_MapSizeY |= fgetc(fp) << 24;
	
	// }bvf[^zm
	int i;
	int size = sizeof(unsigned long) * m_MapSizeX * m_MapSizeY;
	SAFE_FREE(m_pMapData);
	m_pMapData = (unsigned long *)malloc(size);
	ZeroMemory(m_pMapData, size);
	
	// zɊi[
	unsigned char *p = (unsigned char *)m_pMapData;
	
	for (i = 0; i < size; i++) {
		if (feof(fp)){
			SAFE_FREE(m_pMapData);
			return false;
		}
		*p = fgetc(fp);
		p++;
	}
	
	fclose(fp);
	
	return TRUE;
}// Load_MapData

//	}bv̕\
//@bmBack			: [IN][OUT]	obNobt@
//@bmMapChip		: [IN]		}bv`bv
//@bmLyerChip		: [IN]		C[`bv
//@scr_x, scr_y	: [IN]		}bṽXN[ʒu
//@x, y			: [IN]		}bv̍W
BOOL CGm32::DrawMap(CBm32 &bmBack, CBm32 &bmMapChip, CBm32 &bmLyerChip, int scr_x, int scr_y, int x, int y)
{
	if(!m_pMapData)
		return FALSE;
	
	if(scr_x < 0)
		scr_x = 0;
	if(scr_y < 0)
		scr_y = 0;
	
	int i, j, pos;
	int map_left   = scr_x / CHIP_SIZE;
	int map_right  = map_left + m_MapWidth;
	int map_top    = scr_y / CHIP_SIZE;
	int map_bottom = map_top + m_MapHeight;
	
	if (scr_x % CHIP_SIZE)
		map_right  ++;
	if (scr_y % CHIP_SIZE)
		map_bottom ++;
	if (map_right  > m_MapSizeX)
		map_right  = m_MapSizeX;
	if (map_bottom > m_MapSizeY)
		map_bottom = m_MapSizeY;
	
	for (i = map_top; i < map_bottom; i++) {
		for (j = map_left; j < map_right; j++) {
			pos = i * m_MapSizeX + j;
			// wi\
			int idx = m_pMapData[pos] & BYTE_MASK_BG;
			int x2 = j * CHIP_SIZE + x - scr_x;
			int y2 = i * CHIP_SIZE + y - scr_y;
			int u2 = idx % CHIP_IMG_ROW;
			int v2 = idx / CHIP_IMG_ROW;
			int u = u2 * CHIP_SIZE;
			int v = v2 * CHIP_SIZE;
			bmMapChip.Draw(bmBack, x2, y2, u, v, CHIP_SIZE, CHIP_SIZE);
			// C[\
			if(HIT_SIZE == 32 && !(m_pMapData[pos] & BIT_MASK_PRI16_0)){
				idx = (m_pMapData[pos] & BYTE_MASK_LYER) >> BYTE_SHIFT_LYER;
				if(idx != 0) { // `bvhcÔ͍ߕ`悵Ȃ
					int u = (idx % CHIP_IMG_ROW) * CHIP_SIZE;
					int v = (idx / CHIP_IMG_ROW) * CHIP_SIZE;
					if(bmLyerChip.GetColorKeyFlag())
						bmLyerChip.DrawKey(bmBack, x2, y2, u, v, CHIP_SIZE, CHIP_SIZE);
					else
						bmLyerChip.Draw32(bmBack, x2, y2, u, v, CHIP_SIZE, CHIP_SIZE, 255);
				}
			}
		}
	}
	
	return TRUE;
}// DrawMap

//	vCIeB̍C[`bv̕\
//@bmBack			: [IN][OUT]	obNobt@
//@bmLyerChip		: [IN]		C[`bv
//@scr_x, scr_y	: [IN]		}bṽXN[ʒu
//@x, y			: [IN]		}bv̍W
BOOL CGm32::DrawPriorityMap(CBm32 &bmBack, CBm32 &bmLyerChip, int scr_x, int scr_y, int x, int y)
{
	if(!m_pMapData)
		return FALSE;
	
	if(scr_x < 0)
		scr_x = 0;
	if(scr_y < 0)
		scr_y = 0;
	
	int i, j, pos;
	int map_left   = scr_x / CHIP_SIZE;
	int map_right  = map_left + m_MapWidth;
	int map_top    = scr_y / CHIP_SIZE;
	int map_bottom = map_top + m_MapHeight;
	
	if (scr_x % CHIP_SIZE)
		map_right  ++;
	if (scr_y % CHIP_SIZE)
		map_bottom ++;
	if (map_right  > m_MapSizeX)
		map_right  = m_MapSizeX;
	if (map_bottom > m_MapSizeY)
		map_bottom = m_MapSizeY;
	
	for (i = map_top; i < map_bottom; i++) {
		for (j = map_left; j < map_right; j++) {
			pos = i * m_MapSizeX + j;
			// C[\
			if(HIT_SIZE == 32 && (m_pMapData[pos] & BIT_MASK_PRI16_0)){
				int idx = (m_pMapData[pos] & BYTE_MASK_LYER) >> BYTE_SHIFT_LYER;
				if(idx != 0) { // `bvhcÔ͍ߕ`悵Ȃ
					int x2 = j * CHIP_SIZE + x - scr_x;
					int y2 = i * CHIP_SIZE + y - scr_y;
					int u = (idx % CHIP_IMG_ROW) * CHIP_SIZE;
					int v = (idx / CHIP_IMG_ROW) * CHIP_SIZE;
					if(bmLyerChip.GetColorKeyFlag())
						bmLyerChip.DrawKey(bmBack, x2, y2, u, v, CHIP_SIZE, CHIP_SIZE);
					else
						bmLyerChip.Draw32(bmBack, x2, y2, u, v, CHIP_SIZE, CHIP_SIZE, 255);
				}
			}
		}
	}
	
	return TRUE;
}// DrawPriorityMap

//	蔻̕\
//@bmBack			: [IN][OUT]	obNobt@
//@scr_x, scr_y	: [IN]		}bṽXN[ʒu
//@x, y			: [IN]		}bv̍W
//@red				: [IN]		ԐF
//@green			: [IN]		ΐF
//@blue 			: [IN]		F
BOOL CGm32::DrawHitJudge(CBm32 &bmBack, int scr_x, int scr_y, int x, int y, BYTE red, BYTE green, BYTE blue)
{
	if(!m_pMapData)
		return FALSE;
	
	if(scr_x < 0)
		scr_x = 0;
	if(scr_y < 0)
		scr_y = 0;
	
	int i, j;
	int map_left   = scr_x / CHIP_SIZE;
	int map_right  = map_left + m_MapWidth;
	int map_top    = scr_y / CHIP_SIZE;
	int map_bottom = map_top + m_MapHeight;
	
	if (scr_x % CHIP_SIZE)
		map_right  ++;
	if (scr_y % CHIP_SIZE)
		map_bottom ++;
	if (map_right  > m_MapSizeX)
		map_right  = m_MapSizeX;
	if (map_bottom > m_MapSizeY)
		map_bottom = m_MapSizeY;
	
	// 蔻̕`
	if(m_pMapData){
		for (i = map_top; i < map_bottom; i++) {
			for (j = map_left; j < map_right; j++) {
				if(m_pMapData[i * m_MapSizeX + j] & BIT_MASK_HIT16_0){
					int x2 = j * HIT_SIZE + x - scr_x;
					int y2 = i * HIT_SIZE + y - scr_y;
					bmBack.AddColor(red, green, blue, x2, y2, HIT_SIZE, HIT_SIZE);
				}
			}
		}
	}
	
	return TRUE;
}// DrawHitJudge

// CGm32 ̃fXgN^
CGm32::~CGm32()
{
	SAFE_FREE(m_pMapData);
}// ~CGm32
